/*************************************************************
*
* Description:
*   ARC-EM - Example program using Synopsys MIDE
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#include "MIDE/s/sec_interface.h"

typedef struct port_ctrl {
    U32 DR;
    U32 DDR;
    U32 CTRL;
} PORT_CTRL;
typedef volatile struct dw_gpio_reg {
    PORT_CTRL SWPORTS[4];
} *LED_REG_PTR;
#define REGBASE_GPIO0           (0x00002000U)   /*!< GPIO 0 Onboard */
LED_REG_PTR led_reg_ptr = (LED_REG_PTR)(0xF0000000U + REGBASE_GPIO0);

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif
OS_SEM TEEsem;

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

/***************************************************************/

#ifdef _OS_ARC_EM_

 #define INT_CTRL_IE         (1 << 4)
 #define TIMER_CTRL_IE       (1 << 0)
 #define TIMER_CTRL_NH       (1 << 1)
 #define TIMER_CTRL_IP       (1 << 3)
 #define EXC_NO_TIMER0        16
 #define TIMER0_FREQ          25000000

 void SysTick_init(void)
 {
    _sr(0, ARC_REG_TIMER0_CTRL);
    _sr(TIMER0_FREQ / OS_TIMER_TICKS_PER_SEC, ARC_REG_TIMER0_LIMIT);
    _sr(0, ARC_REG_TIMER0_CNT);
    _sr(TIMER_CTRL_IE | TIMER_CTRL_NH, ARC_REG_TIMER0_CTRL);
 }

 void SysTick_update(void)
 {
    _sr(_lr(ARC_REG_TIMER0_CTRL) & ~TIMER_CTRL_IP, ARC_REG_TIMER0_CTRL);
 }

 void LED_init(void)
 {
    led_reg_ptr->SWPORTS[1].DR  = 0xFF;
    led_reg_ptr->SWPORTS[1].DDR = 0xFF;
 }
 void LED_toggle(void)
 {
    if(led_reg_ptr->SWPORTS[1].DR & 1) led_reg_ptr->SWPORTS[1].DR &= ~1;
    else                               led_reg_ptr->SWPORTS[1].DR |=  1;
 }

 void SystemInit(void)
 {
 #ifndef __Xsec_modes
    U32 irq_ctrl = (1 << 13) |         // LP        auto-save/restore code-density registers
                   (1 << 11) |         // U         user-context to user-stack
                   (1 << 10) |         // L         auto-save/restore loop registers
                   (1 <<  9) |         // B         auto-save/restore BLINK
  #ifndef _ARC_RF16
                   (16);               // NR[4:0]   16 pairs -> all of the 32 registers
  #else
                   (8);                // NR[4:0]    8 pairs -> all of the 16 registers
  #endif
    _seti(INT_CTRL_IE | 2);
    _sr(irq_ctrl, ARC_REG_IRQ_CTRL);
 #else
                                       // on SecureShield configurations, the AUX_IRQ_CTRL have to be written
                                       // on secure-startup and the used interrupts (16 & 18) have to be
                                       // configured to be usable here - take care timer-0 interrupt is off !
    _sr(0, ARC_REG_TIMER0_CTRL);
 #endif
 }

#endif

/****************************************************************
 *   Here a blocking semaphore is used as simple test entry-    *
 *   gate for the TEE access. So only one task can execute      *
 *   code in the TEE at a time.                                 *
 *   A mutex can be used too.                                   *
 *                                                              *
 *   Inside the TEE code/functions no pC/OS is supported.       *
 *   This TEE code/functions have to be OS-less functionality.  *
 ***************************************************************/
const U08 d[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
      U08 r[256];

/*
 * - here on the secure-shield use-case, the "TaskID" should be a parameter
 *   of the "secure_save()" and "secure_restore()" test-functions instead
 *   of calling a none-secure function like "TEE_getTaskID()" and all their
 *   code inside.
 *
 * - to get this TEE entries called from a kernel working fine, you have to use
 *   the compiler-switch "-Hsdata0" to take care the "GP"-register is'nt used for
 *   global variables on none-secure side. The GP-register is invalid / zero in
 *   none-secure Interrupt service routines if the secure-shield was active executing
 *   code at this time.
 */

void TEE_entry_1(void)
{
    (void)OS_SemPend(&TEEsem, OS_SUSPEND);  // alloc TEE access
    secure_save(OS_TaskGetID(OS_PRIO_SELF), d, sizeof(d));              // -> stay some time in the TEE
    (void)OS_SemPost(&TEEsem);              // free  TEE access
}

void TEE_entry_2(void)
{
    int i, l;
    
    (void)OS_SemPend(&TEEsem, OS_SUSPEND);  // alloc TEE access
    l = secure_restore(OS_TaskGetID(OS_PRIO_SELF), r);                  // -> stay some time in the TEE
    (void)OS_SemPost(&TEEsem);              // free  TEE access
    for (i = 0; i < l; i++) {
        if (r[i] != d[i]) break;
    }
    if (i < l)
        while(1);
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif
    OS_SemInit(&TEEsem, 1);

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical ARC-EM pC/OS ISR code behind the assembler entry code
{                                                          // (OS_IntEnter & OS_IntExit are done in the assembler entry code)
    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
        
        // ---- TEE access ----
        TEE_entry_2();
        // --------------------
        
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;

    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------

        // ---- TEE access ----
        TEE_entry_1();
        // --------------------
    }
}
