/*************************************************************
*
* Description:
*   LPC4337-M0 - dual-core AMP Example program using Keil
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

/**********************************
* user LEDs                       *
* P6_12 - LED1 green              *
* P6_11 - LED2 yellow             *
* P6_10 - LED3 red                *
* P6_9  - LED4 orange             *
**********************************/

#define LED_M0_PORT
#define LED_M0_PIN

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
OS_STK_TYPE IStk[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;

OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;

void  ROnceTimer1CB(void *tdata);
void  ROnceTimer2CB(void *tdata);
void  LedTimerCB(void *tdata);

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);
void  ITask(void *Tdata);

void SysTick_init(void)
{
    SystemCoreClockUpdate();
    LPC_RITIMER->COMPVAL = SystemCoreClock / OS_TIMER_TICKS_PER_SEC;
    LPC_RITIMER->COUNTER = 0;
    LPC_RITIMER->CTRL    = (1 << 3) | (1 << 2) | (1 << 1);
    NVIC_EnableIRQ(M0_RITIMER_OR_WWDT_IRQn);                           
}

void SysTick_update(void)
{
    LPC_RITIMER->CTRL |= 1;
}

void M0_RITIMER_OR_WWDT_IRQHandler(void)                    // typical Cortex-M0 RIT pC/OS ISR
{
    OS_IntEnter();

    // ---- HW specific ----
    SysTick_update();
    // ---------------------

    OSTimerTick();                                         // call pC/OS-function (timers)
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
    OS_IntExit();
}

U08 LED;
void LED_init(void)
{
    LPC_SCU->SFSP6_11      = 0x00000028;
    LPC_GPIO_PORT->DIR[3] |= (1 << 7);
    LPC_GPIO_PORT->SET[3]  = (1 << 7);
    LED = 0;
}

void LED_toggle(void)
{
    if(LED) LPC_GPIO_PORT->CLR[3]  = (1 << 7);
    else    LPC_GPIO_PORT->SET[3]  = (1 << 7);
    LED = (LED)? 0:1;
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
//    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(ITask, NULL, &IStk, TASK_STK_SIZE,  9);
    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);

    OS_SemInit(&LedSem, 0);

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // un-used
{
    while(1);
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

void Task3(void *Tdata)
{
    Tdata = Tdata;
 
    // ---- HW specific ----
    LED_init();
    // ---------------------

    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);

    while(1) {
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timer-triggered signal

        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}

/***************************************************************/
OS_PACKED typedef struct ICC_cmd {                         // ICC communication data structure
    U08 cmd;
    U16 timeout;
    U08 reserved[5];
} OS_PACKED_ATTR icc_cmd_t;

void ITask(void *Tdata)                                    // ICC AMP signal/message from M7 collecting task
{
    icc_cmd_t icc;
    U16       len;
    U08       returnOk;

    OS_IccInit();
    while(1) {
        returnOk = OS_IccPend((U08 *)&icc, &len, OS_SUSPEND);
        if (returnOk) continue;
        switch (icc.cmd) {
        case 1:
            OS_TimerStop(&LedTimer);                       // stop LED-timer
            break;
        case 2:
            OS_TimerDelete(&ROnceTimer1);    
            OS_TimerStop(&ROnceTimer2);    
            OS_TimerStart(&LedTimer, 0);                   // restart LED-timer with org time
            OS_TimerCreate(&ROnceTimer1, icc.timeout, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
            OS_TimerStart(&ROnceTimer2, 0);                // start both run-once (one auto-clear) timer again
            break;
        case 3:
            OS_SemPost(&LedSem);
            break;
        default:
            break;
        }
    }
}

void ROnceTimer1CB(void *tdata)
{
    icc_cmd_t icc;

    tdata = tdata;
    icc.cmd = 1;                                           // AMP signal/message to M7
    OS_IccPost((U08 *)&icc, 1, OS_NO_SUSP);
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerDelete(&ROnceTimer1);    
    OS_TimerStop(&ROnceTimer2);    
}

void LedTimerCB(void *tdata)
{
    icc_cmd_t icc;

    tdata = tdata;
    icc.cmd = 3;                                           // AMP signal/message to M7
    OS_IccPost((U08 *)&icc, 1, OS_NO_SUSP);
}
