/*************************************************************
*
* Description:
*	STR711 - Example program using IAR
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"
#include "../../../inc/STR71x/STR71x_Regs.h"               // from CrossWorks

#define  CPU_CRISTAL_CLK  4000000UL                        // processor crystal-clock in Hz

#define  TMR0_PRESCALER       0xFF
#define  TMR0_INT             0


#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

/*
************************************************************
*              INT-vector table in RAM
************************************************************
*/
void *IntVectorTable[32];

void OS_Dummy_ISR(void)
{
    while(1);
}

void OS_InitVecTab(void)
{
    U08 i;
    
    STR_EIC_ICR = STR_EIC_ICR_IRQ_EN;
    for(i=0; i< (sizeof(IntVectorTable)/sizeof(void *)); i++)
        IntVectorTable[i] = (void *)&OS_Dummy_ISR;
}

void OS_SetVec(U16 vec_num, void *ptr)
{
    IntVectorTable[vec_num] = ptr;
    STR_EIC_SIRn[vec_num] = ((U32)vec_num) << 16;
}

/*
************************************************************
*                 test-application
************************************************************
*/
int main(void)
{
    while((STR_PRCCU_PWRCR & STR_PRCCU_PWRCR_VROK) == 0);  // wait for stable Main Voltage Regulator
    STR_PRCCU_PLL1CR |= STR_PRCCU_PLL1CR_FREF_RANGE;   // CK > 3MHz
    STR_PRCCU_PLL1CR |= STR_PRCCU_PLL1CR_FREEN;        // disable free running
    STR_PRCCU_PLL1CR |= STR_PRCCU_PLL1CR_MX0;          // PLL1*12
    STR_PRCCU_PLL1CR &= ~STR_PRCCU_PLL1CR_MX1;
    STR_PRCCU_PLL1CR &= ~STR_PRCCU_PLL1CR_DX_MASK;     // divide PLL1 by 1
    STR_PRCCU_CFR    &= ~STR_PRCCU_CFR_DIV2;           // CK=CK2
    STR_PRCCU_CFR    |= STR_PRCCU_CFR_CK2_16;          // select PLL1
    STR_PRCCU_CFR    |= STR_PRCCU_CFR_CSU_CKSEL;
    while ((STR_PRCCU_CFR & STR_PRCCU_CFR_LOCK) == 0); // wait for lock PLL1
    STR_PRCCU_PDIVR &= ~(STR_PRCCU_PDIVR_FACT1_MASK | STR_PRCCU_PDIVR_FACT2_MASK);  //PCLK1 = PCLK2 = RCLK 48MHz

    OS_InitVecTab();
    
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif
    
    OS_Start();
    
    while(1);
}

U16 Tmr0_Value;

void OSTickISR(void)                                       // typical STR71x pC/OS ISR
{
    STR_TIM0_SR   &= ~STR_TIM0_SR_OCFA;                    // clear OCFA-bit
    STR_TIM0_OCAR  = Tmr0_Value;                           // reload timer-end (compare) value
    STR_TIM0_CNTR  = 0;                                    // restart the timer

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the system tick
    U32 s_clksrc;
    
    OS_SystemTicksCnt = 0;
    s_clksrc  = CPU_CRISTAL_CLK;                                       // set start-clk (cristal-clk)
    if(STR_PRCCU_CFR & STR_PRCCU_CFR_DIV2) s_clksrc /= 2U;             // is DIV2 active
    if(STR_PRCCU_CFR & STR_PRCCU_CFR_CSU_CKSEL) {                      // if PLL enabled
        while(!(STR_PRCCU_CFR & STR_PRCCU_CFR_LOCK));                  // be shure PLL is locked
        switch((STR_PRCCU_PLL1CR & STR_PRCCU_PLL1CR_MX_MASK) >> 4) {   // mul clk with PLL-multiplier  (defined at startup)
            case 0:  s_clksrc *= 20U; break;
            case 1:  s_clksrc *= 12U; break;
            case 2:  s_clksrc *= 24U; break;
            case 3:  
            default: s_clksrc *= 16U; break;
        }
        s_clksrc /= ((STR_PRCCU_PLL1CR & STR_PRCCU_PLL1CR_DX_MASK) + 1); // div clk with PLL-dividier  (defined at startup)
    } else {
        if(!(STR_PRCCU_CFR & STR_PRCCU_CFR_CK2_16)) {                    // if no PLL in use, check the CLK2/16 use
            s_clksrc /= 16U;
        }
    }
    s_clksrc /= (1U << (STR_PRCCU_PDIVR & STR_PRCCU_PDIVR_FACT2_MASK));  // PDIV for PCLK2 (APB2)
  
    Tmr0_Value = (s_clksrc / (TMR0_PRESCALER + 1)) / OS_TIMER_TICKS_PER_SEC;   // calculate value for found PCLK2
    OS_SetVec((U16)TMR0_INT, (void *)&OSTickISR);    
    STR_EIC_SIRn[TMR0_INT] |= 0x00000001;                  // set prio
    STR_EIC_IER    = (1 << TMR0_INT);                      // enable INTs for Timer0
    
    STR_TIM0_CR2   = STR_TIM0_CR2_OCAIE | TMR0_PRESCALER;  // enable OCAI and set prescaler
    STR_TIM0_CR1   = STR_TIM0_CR1_EN | STR_TIM0_CR1_OCAE;  // enable & OCAE
    STR_TIM0_OCAR  = Tmr0_Value;                           // load timer-end (compare) value
    STR_TIM0_CNTR  = 0;                                    // start the timer
}

void Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Taskswitch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                  // stope LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);              // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);           // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;                                         // Configure the LED on pin P1.8
  
    STR_IOPORT1_PC0 &= ~(1 << 8);                          // Configure  P1.8 pin to output OD
    STR_IOPORT1_PC1 &= ~(1 << 8);
    STR_IOPORT1_PC2 |=  (1 << 8);
    STR_IOPORT1_PD  &= ~(1 << 8);                          // switch ON (control)
#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif    

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        if(STR_IOPORT1_PD & (1 << 8)) STR_IOPORT1_PD &= ~(1 << 8); 
        else                          STR_IOPORT1_PD |= (1 << 8);         
    }
}
